const hasOwnProperty = Object.prototype.hasOwnProperty;
function preserve(obj) {
    const result = Object.create(null);
    for (const key in obj) {
        if (hasOwnProperty.call(obj, key)) {
            result[key] = obj[key];
        }
    }
    return Object.freeze(result);
}
export function createMatcher(kindKey, kindDict) {
    const kinds = preserve(kindDict);
    function match(matched, options, defaultTo) {
        const kind = matched[kindKey];
        const handler = hasOwnProperty.call(options, kind) ? options[kind] : null;
        if (handler) {
            return handler(kinds[kind](matched));
        }
        if (defaultTo) {
            return defaultTo();
        }
        throw new TypeError("unknown kind");
    }
    return match;
}
